#
# Copyright (C) 2021 diabible.com
#
# This file is part of Diabible Parser.
#
# Diabible Parser is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Diabible Parser is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Diabible Parser. If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Michal Mocnak <michal@marigan.net>
#

require 'zip'
require 'nokogiri'
require 'open-uri'
require 'diabible/parser/helpers'

module Diabible
  module Parser

    class Document
      include Helpers::Info
      include Helpers::Metadata
      include Helpers::Text

      # default constructor
      def initialize(document, info: true, text: true, limit: 0)
        Zip::File.open(URI.open(document)) do |zip_file|
          # open content xml
          @document = Nokogiri::XML(zip_file.read("word/document.xml"))
        end
        # remove namespaces if requested
        @document.remove_namespaces!
        # defaults
        @text = ''
        @metadata = []
        @limit = limit
        # parse
        parse_info if info
        parse_text if text
      end

      def metadata
        @metadata
      end

      def text
        @text
      end

      def to_s
        @text
      end

      def to_json
        {
          text: @text,
          metadata: @metadata
        }
      end

      private

      def count
        @count ||= 0
        @count += 1
      end

      # get text content of all nodes matched by xpath
      def content(node, xpath)
        # content container
        content = ''
        # parse
        node.xpath(xpath).each do |element|
          content += element.content
        end
        # return container
        content
      end
    end
  end
end
