#
# Copyright (C) 2021 diabible.com
#
# This file is part of Diabible Parser.
#
# Diabible Parser is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Diabible Parser is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Diabible Parser. If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Michal Mocnak <michal@marigan.net>
#

module Diabible
  module Parser
    module Helpers
      module Metadata

        # add meta hash
        def add_metadata(group, type, value, position = nil)
          # get index
          index = count
          # normally add a new meta hash with uniqe name
          @metadata << { name: "#{index}_#{group}_#{type}", group: group, type: type, index: index, value: value, position: position }
        end

        # get metadata array per group or type
        def get_metadata(group: nil, type: nil, filter: :exclusive, filters: [])
          if group and type
            return metadata_query(filter: filter, filters: filters) do |m|
              m[:group] == group and m[:type] == type
            end
          elsif group
            return metadata_query(filter: filter, filters: filters) do |m|
              m[:group] == group
            end
          elsif type
            return metadata_query(filter: filter, filters: filters) do |m|
              m[:type] == type
            end
          else
            return metadata_query(filter: filter, filters: filters) do |m|
              true
            end
          end
        end

        def metadata_query(filter: :exclusive, filters: [])
          return @metadata.select do |m|
            if filters.empty?
              yield m
            else
              # get filters intersection
              filtered = filters & [m[:type], m[:group]]
              # resolution
              (yield m) and (filter == :exclusive ? filtered.empty? : !filtered.empty?)
            end
          end
        end
      end
    end
  end
end
