#
# Copyright (C) 2021 diabible.com
#
# This file is part of Diabible Parser.
#
# Diabible Parser is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Diabible Parser is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Diabible Parser. If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Michal Mocnak <michal@marigan.net>
#

require 'diabible/parser/helpers/constants'

module Diabible
  module Parser
    module Helpers
      module Text
        include Diabible::Parser::Helpers::Constants

        private

        # parse main text entries
        def parse_text
          # iterate through all paragraphs without style
          @document.xpath("//body/p").each do |p|
            # store pStyle value
            pStyle = p.xpath(".//pStyle/@val")
            # prepare group
            group = pStyle.first.value unless pStyle.empty?
            group ||= NONE
            # position in for paragraph against plain text
            paragraph_in = @text.length
            # iterate over r blocks
            p.xpath(".//r").each do |r|
              # store rstyle value
              rStyle = r.xpath(".//rStyle/@val")
              # if empty then it's a plain text without style
              if (rStyle.empty? or rStyle.first.value === 'Text') and pStyle.empty?
                # update text object
                @text += content(r, ".//t")
              else
                # otherwise create metadata field
                # prepare type
                type = rStyle.first.value unless rStyle.empty?
                type ||= NONE
                # prepare value
                value = content(r, ".//t")
                # add into container
                add_metadata(group, type, value, { in: @text.length })
              end
            end
            # position out for paragraph
            paragraph_out = @text.length
            # filter paragraphs without content
            add_metadata(PARAGRAPH, group, NONE, { in: paragraph_in, out: paragraph_out })
            # limit check
            if @limit > 0
              if @metadata.size >= @limit
                # when meets limit break
                break
              end
            end
          end
        end
      end
    end
  end
end
